﻿using System;
using System.Numerics;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Media;

namespace TestProject
{
    /// <summary>
    /// 메인 페이지
    /// </summary>
    public sealed partial class MainPage : Page
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainPage()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            #region 윈도우 크기를 설정한다.

            double width  = 800d;
            double height = 600d;

            double dpi = (double)DisplayInformation.GetForCurrentView().LogicalDpi;

            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;

            Size windowSize = new Size(width * 96d / dpi, height * 96d / dpi);

            ApplicationView.PreferredLaunchViewSize = windowSize;

            Window.Current.Activate();

            ApplicationView.GetForCurrentView().TryResizeView(windowSize);

            #endregion
            #region 윈도우 제목을 설정한다.

            ApplicationView.GetForCurrentView().Title = "RadialGradientBrush 엘리먼트 사용하기";

            #endregion

            Loaded += Page_Loaded;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 페이지 로드시 처리하기 - Page_Loaded(sender, e)

        /// <summary>
        /// 페이지 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
            this.mappingModeComboBox.SelectionChanged  += mappingModeComboBox_SelectionChanged;
            this.spreadMethodComboBox.SelectionChanged += spreadMethodComboBox_SelectionChanged;

            InitializeSlider();
        }

        #endregion

        #region 스프레드 메소드 콤보 박스 선택 변경시 처리하기 - spreadMethodComboBox_SelectionChanged(sender, e)

        /// <summary>
        /// 스프레드 메소드 콤보 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void spreadMethodComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.radialGradientBrush.SpreadMethod = Enum.Parse<GradientSpreadMethod>(this.spreadMethodComboBox.SelectedValue.ToString());
        }

        #endregion
        #region 매핑 모드 콤보 박스 선택 변경시 처리하기 - mappingModeComboBox_SelectionChanged(sender, e)

        /// <summary>
        /// 매핑 모드 콤보 박스 선택 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void mappingModeComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            this.radialGradientBrush.MappingMode = Enum.Parse<BrushMappingMode>(this.mappingModeComboBox.SelectedValue.ToString());

            InitializeSlider();
        }

        #endregion
        #region 슬라이더 값 변경시 처리하기 - slider_ValueChanged(sender, e)

        /// <summary>
        /// 슬라이더 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void slider_ValueChanged(object sender, RangeBaseValueChangedEventArgs e)
        {
            this.radialGradientBrush.Center = new Point(this.centerXSlider.Value, this.centerYSlider.Value);

            this.radialGradientBrush.RadiusX = this.radiusXSlider.Value;
            this.radialGradientBrush.RadiusY = this.radiusYSlider.Value;

            this.radialGradientBrush.GradientOrigin = new Point(this.gradientOriginXSlider.Value, this.gradientOriginYSlider.Value);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 슬라이더 초기화하기 - InitializeSlider()

        /// <summary>
        /// 슬라이더 초기화하기
        /// </summary>
        private void InitializeSlider()
        {
            Size rectangleSize = this.rectangle.ActualSize.ToSize();

            if(this.radialGradientBrush.MappingMode == BrushMappingMode.Absolute)
            {
                double width          = rectangleSize.Width;
                double halfWidth      = rectangleSize.Width  / 2;
                double stepFrequencyX = rectangleSize.Width  / 50;
                double stepFrequencyY = rectangleSize.Height / 50;

                this.centerXSlider.Maximum       = width;
                this.centerXSlider.Value         = halfWidth;
                this.centerXSlider.StepFrequency = stepFrequencyX;

                this.centerYSlider.Maximum       = width;
                this.centerYSlider.Value         = halfWidth;
                this.centerYSlider.StepFrequency = stepFrequencyY;

                this.radiusXSlider.Maximum       = width;
                this.radiusXSlider.Value         = halfWidth;
                this.radiusXSlider.StepFrequency = stepFrequencyX;

                this.radiusYSlider.Maximum       = width;
                this.radiusYSlider.Value         = halfWidth;
                this.radiusYSlider.StepFrequency = stepFrequencyY;

                this.gradientOriginXSlider.Maximum       = width;
                this.gradientOriginXSlider.Value         = halfWidth;
                this.gradientOriginXSlider.StepFrequency = stepFrequencyX;

                this.gradientOriginYSlider.Maximum       = width;
                this.gradientOriginYSlider.Value         = halfWidth;
                this.gradientOriginYSlider.StepFrequency = stepFrequencyY;
            }
            else
            {
                this.centerXSlider.Maximum       = 1.0;
                this.centerXSlider.Value         = 0.5;
                this.centerXSlider.StepFrequency = 0.02;

                this.centerYSlider.Maximum       = 1.0;
                this.centerYSlider.Value         = 0.5;
                this.centerYSlider.StepFrequency = 0.02;

                this.radiusXSlider.Maximum       = 1.0;
                this.radiusXSlider.Value         = 0.5;
                this.radiusXSlider.StepFrequency = 0.02;

                this.radiusYSlider.Maximum       = 1.0;
                this.radiusYSlider.Value         = 0.5;
                this.radiusYSlider.StepFrequency = 0.02;

                this.gradientOriginXSlider.Maximum       = 1.0;
                this.gradientOriginXSlider.Value         = 0.5;
                this.gradientOriginXSlider.StepFrequency = 0.02;

                this.gradientOriginYSlider.Maximum       = 1.0;
                this.gradientOriginYSlider.Value         = 0.5;
                this.gradientOriginYSlider.StepFrequency = 0.02;
            }
        }

        #endregion
    }
}